@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2014-2016 Yubico AB
+ * Copyright (c) 2014-2020 Yubico AB
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -43,7 +43,7 @@
 #include <stddef.h>
 #include <stdbool.h>
 
-#include <ykpiv-version.h>
+#include "ykpiv-config.h"
 
 #ifdef __cplusplus
 extern "C"
@@ -67,10 +67,10 @@ extern "C"
     YKPIV_INVALID_OBJECT = -11,
     YKPIV_ALGORITHM_ERROR = -12,
     YKPIV_PIN_LOCKED = -13,
-
     YKPIV_ARGUMENT_ERROR = -14, //i.e. invalid input argument
     YKPIV_RANGE_ERROR = -15, //i.e. value range error
-    YKPIV_NOT_SUPPORTED = -16
+    YKPIV_NOT_SUPPORTED = -16,
+    YKPIV_PCSC_SERVICE_ERROR = -17,
   } ykpiv_rc;
 
   typedef void* (*ykpiv_pfn_alloc)(void* alloc_data, size_t size);
@@ -89,9 +89,11 @@ extern "C"
   ykpiv_rc ykpiv_init(ykpiv_state **state, int verbose);
   ykpiv_rc ykpiv_init_with_allocator(ykpiv_state **state, int verbose, const ykpiv_allocator *allocator);
   ykpiv_rc ykpiv_done(ykpiv_state *state);
+  ykpiv_rc ykpiv_validate(ykpiv_state *state, const char *wanted);
   ykpiv_rc ykpiv_connect(ykpiv_state *state, const char *wanted);
   ykpiv_rc ykpiv_list_readers(ykpiv_state *state, char *readers, size_t *len);
   ykpiv_rc ykpiv_disconnect(ykpiv_state *state);
+  ykpiv_rc ykpiv_translate_sw(int sw);
   ykpiv_rc ykpiv_transfer_data(ykpiv_state *state, const unsigned char *templ,
                                const unsigned char *in_data, long in_len,
                                unsigned char *out_data, unsigned long *out_len, int *sw);
@@ -118,8 +120,11 @@ extern "C"
                              int *tries);
   ykpiv_rc ykpiv_fetch_object(ykpiv_state *state, int object_id,
                               unsigned char *data, unsigned long *len);
+  ykpiv_rc ykpiv_authenticate2(ykpiv_state *state, unsigned const char *key, size_t len);
   ykpiv_rc ykpiv_set_mgmkey2(ykpiv_state *state, const unsigned char *new_key,
                              const unsigned char touch);
+  ykpiv_rc ykpiv_set_mgmkey3(ykpiv_state *state, const unsigned char *new_key, size_t len, unsigned char algorithm,
+                             unsigned char touch);
   ykpiv_rc ykpiv_save_object(ykpiv_state *state, int object_id,
                              unsigned char *indata, size_t len);
   ykpiv_rc ykpiv_import_private_key(ykpiv_state *state, const unsigned char key, unsigned char algorithm,
@@ -131,6 +136,7 @@ extern "C"
                                     const unsigned char *ec_data, unsigned char ec_data_len,
                                     const unsigned char pin_policy, const unsigned char touch_policy);
   ykpiv_rc ykpiv_attest(ykpiv_state *state, const unsigned char key, unsigned char *data, size_t *data_len);
+  ykpiv_rc ykpiv_get_metadata(ykpiv_state *state, const unsigned char key, unsigned char *data, size_t *data_len);
 
   /**
    * Return the number of PIN attempts remaining before PIN is locked.
@@ -194,6 +200,19 @@ extern "C"
    */
   ykpiv_rc ykpiv_verify_select(ykpiv_state *state, const char *pin, const size_t pin_len, int *tries, bool force_select);
 
+  /**
+   * Get serial number
+   *
+   * The card must be connected to call this function.
+   *
+   * @param state [in] State handle
+   * @param p_serial [out] uint32 to store retrieved serial number
+   *
+   * @return ykpiv_rc error code
+   *
+   */
+  ykpiv_rc ykpiv_get_serial(ykpiv_state *state, uint32_t* p_serial);
+
 
 ////////////////////////////////////////////////////////////////////////////////
 ////////////////////////////////////////////////////////////////////////////////
@@ -251,6 +270,7 @@ extern "C"
 #pragma pack(pop)
 
   typedef enum {
+    YKPIV_CONFIG_MGM_INVALID = -1,
     YKPIV_CONFIG_MGM_MANUAL = 0,
     YKPIV_CONFIG_MGM_DERIVED = 1,
     YKPIV_CONFIG_MGM_PROTECTED = 2
@@ -258,18 +278,29 @@ extern "C"
 
 #pragma pack(push, 1)
   typedef struct _ykpiv_config {
-    uint8_t               protected_data_available;
     uint8_t               puk_blocked;
     uint8_t               puk_noblock_on_upgrade;
     uint32_t              pin_last_changed;
     ykpiv_config_mgm_type mgm_type;
+    size_t                mgm_len;
+    uint8_t               mgm_key[32];
   } ykpiv_config;
 
   typedef struct _ykpiv_mgm {
-    uint8_t data[24];
+    size_t len;
+    uint8_t data[32];
   } ykpiv_mgm;
 #pragma pack(pop)
 
+  typedef struct _ykpiv_metadata {
+    uint8_t algorithm;
+    uint8_t pin_policy;
+    uint8_t touch_policy;
+    uint8_t origin;
+    size_t pubkey_len;
+    uint8_t pubkey[512];
+  } ykpiv_metadata;
+
   /**
    * Free allocated data
    *
@@ -410,6 +441,18 @@ extern "C"
    */
   ykpiv_rc ykpiv_util_get_protected_mgm(ykpiv_state *state, ykpiv_mgm *mgm);
 
+  /**
+   * Update Protected MGM key. Should only be used when mgm_type is YKPIV_CONFIG_MGM_PROTECTED.
+   *
+   * The user pin must be verified to call this function
+   *
+   * @param state State handle
+   * @param mgm   [in] Protected MGM key
+   *
+   * @return ykpiv_rc error code
+   */
+  ykpiv_rc ykpiv_util_update_protected_mgm(ykpiv_state *state, ykpiv_mgm *mgm);
+
   /**
    * Set Protected MGM key
    *
@@ -492,7 +535,7 @@ extern "C"
    * @param state state
    * @param ccc Unique Card ID to set. If NULL, randomly generate.
    *
-   * @return ypiv_rc error code
+   * @return ykpiv_rc error code
    *
    */
   ykpiv_rc ykpiv_util_set_cccid(ykpiv_state *state, const ykpiv_cccid *ccc);
@@ -534,8 +577,8 @@ extern "C"
   ykpiv_rc ykpiv_util_write_mscmap(ykpiv_state *state, ykpiv_container *containers, size_t n_containers);
   ykpiv_rc ykpiv_util_read_msroots(ykpiv_state  *state, uint8_t **data, size_t *data_len);
   ykpiv_rc ykpiv_util_write_msroots(ykpiv_state *state, uint8_t *data, size_t data_len);
-
-
+  ykpiv_rc ykpiv_util_parse_metadata(uint8_t *data, size_t data_len, ykpiv_metadata *metadata);
+  
 ////////////////////////////////////////////////////////////////////////////////
 ////////////////////////////////////////////////////////////////////////////////
 ////////////////////////////////////////////////////////////////////////////////
@@ -550,11 +593,16 @@ extern "C"
 
 #define YKPIV_ALGO_TAG 0x80
 #define YKPIV_ALGO_3DES 0x03
+#define YKPIV_ALGO_AES128 0x08
+#define YKPIV_ALGO_AES192 0x0a
+#define YKPIV_ALGO_AES256 0x0c
 #define YKPIV_ALGO_RSA1024 0x06
 #define YKPIV_ALGO_RSA2048 0x07
 #define YKPIV_ALGO_ECCP256 0x11
 #define YKPIV_ALGO_ECCP384 0x14
 
+#define YKPIV_ALGO_AUTO 0xff
+
 #define YKPIV_KEY_AUTHENTICATION 0x9a
 #define YKPIV_KEY_CARDMGM 0x9b
 #define YKPIV_KEY_SIGNATURE 0x9c
@@ -595,6 +643,9 @@ extern "C"
 #define YKPIV_OBJ_DISCOVERY 0x7e
 #define YKPIV_OBJ_KEY_HISTORY 0x5fc10c
 #define YKPIV_OBJ_IRIS 0x5fc121
+#define YKPIV_OBJ_BITGT 0x7f61
+#define YKPIV_OBJ_SM_SIGNER 0x5fc122
+#define YKPIV_OBJ_PC_REF_DATA 0x5fc123
 
 #define YKPIV_OBJ_RETIRED1  0x5fc10d
 #define YKPIV_OBJ_RETIRED2  0x5fc10e
@@ -619,6 +670,10 @@ extern "C"
 
 #define YKPIV_OBJ_ATTESTATION 0x5fff01
 
+#define TAG_CERT              0x70
+#define TAG_CERT_COMPRESS     0x71
+#define TAG_CERT_LRC          0xFE
+
 #define YKPIV_OBJ_MAX_SIZE 3072
 
 #define YKPIV_INS_VERIFY 0x20
@@ -635,7 +690,10 @@ extern "C"
 #define SW_SUCCESS 0x9000
 #define SW_ERR_SECURITY_STATUS 0x6982
 #define SW_ERR_AUTH_BLOCKED 0x6983
+#define SW_ERR_CONDITIONS_OF_USE 0x6985
 #define SW_ERR_INCORRECT_PARAM 0x6a80
+#define SW_ERR_FILE_NOT_FOUND 0x6a82
+#define SW_ERR_REFERENCE_NOT_FOUND 0x6a88
 /* this is a custom sw for yubikey */
 #define SW_ERR_INCORRECT_SLOT 0x6b00
 #define SW_ERR_NOT_SUPPORTED 0x6d00
@@ -647,6 +705,8 @@ extern "C"
 #define YKPIV_INS_RESET 0xfb
 #define YKPIV_INS_SET_PIN_RETRIES 0xfa
 #define YKPIV_INS_ATTEST 0xf9
+#define YKPIV_INS_GET_SERIAL 0xf8
+#define YKPIV_INS_GET_METADATA 0xf7
 
 #define YKPIV_PINPOLICY_TAG 0xaa
 #define YKPIV_PINPOLICY_DEFAULT 0
@@ -660,23 +720,51 @@ extern "C"
 #define YKPIV_TOUCHPOLICY_ALWAYS 2
 #define YKPIV_TOUCHPOLICY_CACHED 3
 
+#define YKPIV_TOUCHPOLICY_AUTO 255
+
+#define YKPIV_METADATA_ALGORITHM_TAG 0x01 // See values for YKPIV_ALGO_TAG
+
+#define YKPIV_METADATA_POLICY_TAG 0x02 // Two bytes, see values for YKPIV_PINPOLICY_TAG and YKPIV_TOUCHPOLICY_TAG
+
+#define YKPIV_METADATA_ORIGIN_TAG 0x03
+#define YKPIV_METADATA_ORIGIN_GENERATED 0x01
+#define YKPIV_METADATA_ORIGIN_IMPORTED 0x02
+
+#define YKPIV_METADATA_PUBKEY_TAG 0x04 // RSA: DER-encoded sequence N, E; EC: Uncompressed EC point X, Y
+
 #define YKPIV_IS_EC(a) ((a == YKPIV_ALGO_ECCP256 || a == YKPIV_ALGO_ECCP384))
 #define YKPIV_IS_RSA(a) ((a == YKPIV_ALGO_RSA1024 || a == YKPIV_ALGO_RSA2048))
 
+#define YKPIV_MIN_PIN_LEN 6
+#define YKPIV_MAX_PIN_LEN 8
+#define YKPIV_MIN_MGM_KEY_LEN 32
+#define YKPIV_MAX_MGM_KEY_LEN 64
+
 #define YKPIV_RETRIES_DEFAULT 3
 #define YKPIV_RETRIES_MAX 0xff
 
 #define YKPIV_CERTINFO_UNCOMPRESSED 0
 #define YKPIV_CERTINFO_GZIP 1
 
+#define YKPIV_OID_FIRMWARE_VERSION "1.3.6.1.4.1.41482.3.3"
+#define YKPIV_OID_SERIAL_NUMBER "1.3.6.1.4.1.41482.3.7"
+#define YKPIV_OID_USAGE_POLICY "1.3.6.1.4.1.41482.3.8"
+#define YKPIV_OID_FORM_FACTOR "1.3.6.1.4.1.41482.3.9"
+
 #define YKPIV_ATR_NEO_R3 "\x3b\xfc\x13\x00\x00\x81\x31\xfe\x15\x59\x75\x62\x69\x6b\x65\x79\x4e\x45\x4f\x72\x33\xe1"
+#define YKPIV_ATR_NEO_R3_NFC "\x3b\x8c\x80\x01\x59\x75\x62\x69\x6b\x65\x79\x4e\x45\x4f\x72\x33\x58"
 #define YKPIV_ATR_YK4    "\x3b\xf8\x13\x00\x00\x81\x31\xfe\x15\x59\x75\x62\x69\x6b\x65\x79\x34\xd4"
+#define YKPIV_ATR_YK5_P1 "\x3b\xf8\x13\x00\x00\x81\x31\xfe\x15\x01\x59\x75\x62\x69\x4b\x65\x79\xc1"
+#define YKPIV_ATR_YK5    "\x3b\xfd\x13\x00\x00\x81\x31\xfe\x15\x80\x73\xc0\x21\xc0\x57\x59\x75\x62\x69\x4b\x65\x79\x40"
+#define YKPIV_ATR_YK5_NFC "\x3b\x8d\x80\x01\x80\x73\xc0\x21\xc0\x57\x59\x75\x62\x69\x4b\x65\x79\xf9"
 
 #define DEVTYPE_UNKNOWN  0x00000000
 #define DEVTYPE_NEO      0x4E450000 //"NE"
 #define DEVTYPE_YK       0x594B0000 //"YK"
 #define DEVTYPE_NEOr3    (DEVTYPE_NEO | 0x00007233) //"r3"
 #define DEVTYPE_YK4      (DEVTYPE_YK  | 0x00000034) // "4"
+#define DEVTYPE_YK5      (DEVTYPE_YK  | 0x00000035) // "5"
+#define DEVYTPE_YK5      DEVTYPE_YK5 // Keep old typo for backwards compatibility
 
 #ifdef __cplusplus
 }
